/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixUtils.c 7     30/09/06 11:45 Dave $

	Copyright (c) 1990-2005 Tarma Software Research Ltd. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	03-09-2004 20:38

	Implementation of helper functions for Tarma extension DLLs.
****************************************************************************/

// Precompiled headers
#include "TixPrecomp.h"
#pragma hdrstop					// End of precompiled headers

// Project headers
#define _TIX_BUILD				// We're implementing the extension DLL
#include "TixDefs.h"
#include "TixRuntime.h"
#include "TixUtils.h"

// Attributes that must be removed for DeleteFile() to delete a file
#define TIX_ATTR_PROTECTED \
	(FILE_ATTRIBUTE_READONLY | FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN)

//===========================================================================
// Implementation helpers
//
// Various helper functions implemented locally to prevent pulling in the
// C/C++ runtime libs and keep the DLL size as small as possible.
//===========================================================================

//---------------------------------------------------------------------------
// Implementation of logging functions with a minimal set of functionality.

#ifdef TIX_TRACE

#include <stdarg.h>		// Only for macros; does not pull in CRT functions

// Global buffers for trace formatting; this allows them to be larger and
// also keeps them away from the stack in case of overflows.

#define TIXTRACE_BUFFER_SIZE	(16 * 1024)	// Large enough???

static TCHAR *g_pszTraceBuffer;
#ifdef _UNICODE
static ACHAR *g_pszTraceBufferA;
#endif

static HANDLE g_hTraceLog = INVALID_HANDLE_VALUE;

BOOL TIXAPI _TixTraceOpenLog(void)
{
	TCHAR szPathBuf[MAX_PATH + 32];		// 32=room for \ and file name
	SYSTEMTIME st;
	UINT uCount;
	HANDLE hHeap;

	if (g_hTraceLog != INVALID_HANDLE_VALUE)
		return TRUE;					// Already open

	// Build the log file path
	uCount = GetTempPath(MAX_PATH, szPathBuf);
	if (uCount == 0 || uCount >= MAX_PATH) szPathBuf[0] = 0;
	lstrcat(szPathBuf, _T("_Tix.log"));

	// Open the log file
	g_hTraceLog = CreateFile(szPathBuf, GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (g_hTraceLog == INVALID_HANDLE_VALUE)
		return FALSE;

	// Seek to the end for appending
	SetFilePointer(g_hTraceLog, 0, NULL, FILE_END);

	// Allocate global buffers for message formatting.
	hHeap = GetProcessHeap();
	g_pszTraceBuffer = HeapAlloc(hHeap, 0,
		TIXTRACE_BUFFER_SIZE * sizeof(TCHAR));
#ifdef _UNICODE
	g_pszTraceBufferA = HeapAlloc(hHeap, 0, TIXTRACE_BUFFER_SIZE);
#endif

	// Add indication of a new session: time and process info
	GetLocalTime(&st);
	_TixTraceLog(_T("----------[%04u-%02u-%02u %02u:%02u:%02u]----------\r\n"),
		st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
	if (GetModuleFileName(NULL, szPathBuf, MAX_PATH) == 0)
		szPathBuf[0] = 0;
	_TixTraceLog(_T("%08X: %s\r\n"), GetCurrentProcessId(), szPathBuf);

	return TRUE;
}

void TIXAPI _TixTraceCloseLog(void)
{
	HANDLE hHeap;

	if (g_hTraceLog != INVALID_HANDLE_VALUE)
	{
		CloseHandle(g_hTraceLog);
		g_hTraceLog = INVALID_HANDLE_VALUE;
	}

	hHeap = GetProcessHeap();
	if (g_pszTraceBuffer != NULL)
	{
		HeapFree(hHeap, 0, g_pszTraceBuffer);
		g_pszTraceBuffer = NULL;
	}
#ifdef _UNICODE
	if (g_pszTraceBufferA != NULL)
	{
		HeapFree(hHeap, 0, g_pszTraceBufferA);
		g_pszTraceBufferA = NULL;
	}
#endif
}

BOOL TIXAPI _TixTraceLog(const TCHAR *pszFormat, ...)
{
	va_list args;
	DWORD dwWritten;

	if (g_hTraceLog == INVALID_HANDLE_VALUE) return FALSE;
	if (pszFormat == NULL || *pszFormat == 0) return FALSE;

	// Format into a "large enough" (??) buffer. No LF -> CR/LF is done.
	va_start(args, pszFormat);
	wvsprintf(g_pszTraceBuffer, pszFormat, args);
	va_end(args);

	// Write to output. For Unicode builds, we convert to UTF-8 first to
	// preserve as much infor as possible, while still writing mostly text.
#ifdef _UNICODE
	WideCharToMultiByte(CP_UTF8, 0, g_pszTraceBuffer, -1, g_pszTraceBufferA,
		TIXTRACE_BUFFER_SIZE, NULL, NULL);
	return WriteFile(g_hTraceLog, g_pszTraceBufferA,
		lstrlenA(g_pszTraceBufferA), &dwWritten, NULL);
#else
	return WriteFile(g_hTraceLog, g_pszTraceBuffer,
		lstrlen(g_pszTraceBuffer), &dwWritten, NULL);
#endif
}

const TCHAR * TIXAPI _TixGetMessageName(DWORD dwMessageID)
{
	static TCHAR szBuffer[64];
	static const TCHAR *g_ppszMessageNames[] =
	{
		_T("GETVERSION"),
		_T("INITIALIZE"),
		_T("TERMINATE"),
		_T("CHECK_REQUIREMENTS"),
		_T("INIT_REGISTRATION"),
		_T("CHECK_REGISTRATION"),
		_T("APPLY_REGISTRATION"),
		_T("START_PROCESSING"),
		_T("END_PROCESSING"),
		_T("ABORT_PROCESSING"),
		_T("INIT_OPTIONS"),
		_T("CHECK_OPTIONS"),
		_T("START_FILE_INSTALL"),
		_T("END_FILE_INSTALL"),
		_T("START_FILE_REGISTRATION"),
		_T("END_FILE_REGISTRATION"),
		_T("START_SHORTCUT_INSTALL"),
		_T("END_SHORTCUT_INSTALL"),
		_T("START_SYSTEM_INSTALL"),
		_T("END_SYSTEM_INSTALL"),
		_T("START_FILE_REMOVAL"),
		_T("END_FILE_REMOVAL"),
		_T("START_FILE_UNREGISTRATION"),
		_T("END_FILE_UNREGISTRATION"),
		_T("START_SHORTCUT_REMOVAL"),
		_T("END_SHORTCUT_REMOVAL"),
		_T("START_SYSTEM_REMOVAL"),
		_T("END_SYSTEM_REMOVAL"),
		_T("CHECK_UNINSTALL"),
	};

	// Get known names from the list
	if (dwMessageID < __countof(g_ppszMessageNames))
		return g_ppszMessageNames[dwMessageID];

	// Format the rest in a static buffer
	wsprintf(szBuffer, _T("MSG_%u"), dwMessageID);
	return szBuffer;
}

#endif	// TIX_TRACE

//---------------------------------------------------------------------------
// TixMemMove
//
// Copy one block of memory to another; handles overlap.

void TIXAPI TixMemMove(void *pDest, const void *pSrc, SIZE_T uCount)
{
	if (pDest == NULL || pSrc == NULL || pDest == pSrc)
		return;

	// Check for overlap and adjust copy direction accordingly
	if (pDest <= pSrc || ((BYTE *)pDest) >= ((const BYTE *)pSrc) + uCount)
	{
		// Copy low to high
		while (uCount-- > 0)
		{
			*(BYTE *)pDest = *(const BYTE *)pSrc;
			pDest = (BYTE *)pDest + 1;
			pSrc = (const BYTE *)pSrc + 1;
		}
	}
	else
	{
		// Copy high to low
		pDest = (BYTE *)pDest + uCount;
		pSrc = (const BYTE *)pSrc + uCount;

		while (uCount-- > 0)
		{
			pDest = (BYTE *)pDest - 1;
			pSrc = (const BYTE *)pSrc - 1;
			*(BYTE *)pDest = *(const BYTE *)pSrc;
		}
	}
}

//---------------------------------------------------------------------------
// TixStrCopy
//
// Safe string copy function. Returns a pointer to the end of the
// destination string, i.e., to its \0 character.

TCHAR * TIXAPI TixStrCopy(const TCHAR *pszSrc, TCHAR *pszDest,
	TCHAR *pszDestEnd)
{
	if (pszDest >= pszDestEnd)
		return pszDest;
	--pszDestEnd;		// Reserve room for the final \0

#ifdef _MBCS
	while (*pszSrc != 0 && pszDest < pszDestEnd)
	{
		*pszDest++ = *pszSrc++;
		if (*pszSrc != 0 && IsDBCSLeadByte(pszSrc[-1]))
		{
			// Copy trail byte if we still have space; always advance
			if (pszDest < pszDestEnd)
				*pszDest++ = *pszSrc;
			++pszSrc;
		}
	}
#else
	while (*pszSrc != 0 && pszDest < pszDestEnd)
		*pszDest++ = *pszSrc++;
#endif

	// Make sure the string is 0-terminated
	*pszDest = 0;
	return pszDest;
}

//---------------------------------------------------------------------------
// TixMessageBox
//
// Display a message box. Can be called in all situations; will only
// display if not in silent mode.
//
// Returns the message box result; in silent mode, returns nDefaultResult.
//
// NOTE: This message box routine is similar in functionality to the new
// Tin3 facilities. However, the implementation below uses wvsprintf() and
// is potentially vulnerable to buffer overflows. Tin3's implementation
// uses its own safe StrPrintf() implemention.

int __cdecl TixMessageBox(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	UINT uMBFlags,				// [In] Message box flags as per MessageBox()
	int nDefaultResult,			// [In] Default result for silent mode
	LPCTSTR lpFormat,			// [In] wsprintf-style format string
	...							// [In] Optional format args
)
{
	TCHAR szTitleBuf[256];
	TCHAR szMessageBuf[1024];
	va_list args;

	// Get some simple cases out of the way first
	if (pTixRuntime == NULL || TIXRT_IsSilentInstall(pTixRuntime) ||
		lpFormat == NULL)
		return nDefaultResult;

	// Format the message
	va_start(args, lpFormat);
	wvsprintf(szMessageBuf, lpFormat, args);
	va_end(args);

	// Grab the product name
	TIXRT_ResolveSymbols(pTixRuntime, _T("<ProductName>"), szTitleBuf,
		__countof(szTitleBuf));

	// Show the message box
	return MessageBox(TIXRT_GetWindow(pTixRuntime), szMessageBuf,
		szTitleBuf, uMBFlags);
}

//---------------------------------------------------------------------------
// TixLoadStringEx
//
// Load a language-specific string resource. The usual FindResourceEx()
// does not work as-is, because individual strings are stored in string
// tables, and only the string tables can be found directly.
//
// Returns nonzero if the string was found (even if truncated), FALSE if not.

BOOL TIXAPI TixLoadStringEx(
	HINSTANCE hModule,			// [In] Resource module handle
	UINT uStringID,				// [In] String resource ID
	WORD wLanguageID,			// [In] Language ID as per MAKELANGID()
	TCHAR *pszResult,			// [Out] Output buffer
	UINT uResultCount			// [In] Size of pszResult in TCHARs
)
{
	UINT uStringTable, uStringIndex;
	HRSRC hRsrc;
	HGLOBAL hResource;
	WORD *pwPos, *pwEndPos;

	// Weed out useless calls
	if (pszResult == NULL || uResultCount == 0)
		return FALSE;
	*pszResult = 0;				// For early exits

	// Calculate the string table and index for the string. Strings are
	// stored in groups of 16; each group is a STRINGTABLE. The first
	// STRINGTABLE resource has index 1, so offset by 1.
	uStringTable = (uStringID / 16) + 1;
	uStringIndex = uStringID % 16;

	// Try to find the desired string table
	hRsrc = FindResourceEx(hModule, RT_STRING, MAKEINTRESOURCE(uStringTable),
		wLanguageID);
	if (hRsrc == NULL) return FALSE;

	// Load the string table resource
	hResource = LoadResource(hModule, hRsrc);
	if (hResource == NULL) return FALSE;

	// Get a real data pointer
	pwPos = (WORD *)LockResource(hResource);
	if (pwPos == NULL) return FALSE;

	// Try to avoid running off the end
	pwEndPos = (WORD *)((BYTE *)pwPos + SizeofResource(hModule, hRsrc));

	// Find the desired string in the table. They are stored as a WORD-size
	// string length followed by the string characters in Unicode (also
	// WORD-sized); no null terminator.
	while (uStringIndex > 0 && pwPos < pwEndPos)
	{
		// Move past the current string (length + string characters)
		pwPos += *pwPos + 1;
		--uStringIndex;
	}

	// Are we still within the string table?
	if (pwPos >= pwEndPos) return FALSE;

	// We should be at the desired string; copy to result.
	if (*pwPos > 0)
	{
#ifdef _UNICODE
		// Do a straight copy but watch out for overflow. lstrcpyn()
		// always adds a \0 terminator.
		if (*pwPos > uResultCount - 1)
			lstrcpyn(pszResult, pwPos + 1, uResultCount);
		else
			lstrcpyn(pszResult, pwPos + 1, *pwPos);
#else
		// Convert Unicode to ANSI. WideCharToMultiByte() won't
		// add a null terminator, because the Unicode string doesn't
		// have one. Reserve 1 position for that.
		pszResult += WideCharToMultiByte(CP_ACP, 0, pwPos + 1, *pwPos,
			pszResult, uResultCount - 1, NULL, NULL);
		*pszResult = 0;
#endif
	}

	// NOTE: No need to unlock or unload resources in Win32
	return TRUE;
}

//---------------------------------------------------------------------------
// TixDialogBox
//
// Run a modal dialog box. This is similar to DialogBoxParam(), but adds
// language-dependent dialog template look-up.
//
// Returns the result code from the dialog's message loop, or -1 on error.

int TIXAPI TixDialogBox(
	HINSTANCE hModule,			// [In] Module containing the dialog resource
	LPCTSTR lpTemplateName,		// [In] Dialog template name/ID
	WORD wLanguageID,			// [In] Language ID as per MAKELANGID()
	HWND hwndParent,			// [In] Parent window handle
	DLGPROC lpDialogFunc,		// [In] Dialog function
	LPARAM lParam				// [In] Dialog initialization parameter
)
{
	HRSRC hRsrc;
	const DLGTEMPLATE *pTemplate;
	int nResult;

	// Find the requested dialog resource
	hRsrc = FindResourceEx(hModule, RT_DIALOG, lpTemplateName, wLanguageID);
	if (hRsrc == NULL)
	{
		TIX_LOG((_T("%08lX[%u]: Dialog template not found\r\n"),
			(DWORD)lpTemplateName, wLanguageID));
		return -1;
	}

	// Load the dialog template (no need to unlock anything later)
	pTemplate = (const DLGTEMPLATE *)LockResource(LoadResource(hModule, hRsrc));
	if (pTemplate == NULL)
	{
		TIX_LOG((_T("%08lX[%u]: Cannot load dialog template\r\n"),
			(DWORD)lpTemplateName, wLanguageID));
		return -1;
	}

	// Run the dialog modally
	nResult = DialogBoxIndirectParam(hModule, pTemplate, hwndParent,
		lpDialogFunc, lParam);
	TIX_LOG((_T("%08lX[%u]: Dialog returns %d\r\n"),
		(DWORD)lpTemplateName, wLanguageID, nResult));
	return nResult;
}

//---------------------------------------------------------------------------
// TixQueryDirectory
//
// Query the user for a folder path.

int __stdcall __BffCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam,
	LPARAM lpData)
{
	UNUSED(lParam);

	switch (uMsg)
	{
	case BFFM_INITIALIZED:
		// Set initial directory. lpData points to the path
		if (lpData != 0)
			SendMessage(hwnd, BFFM_SETSELECTION, TRUE, lpData);
		break;

	case BFFM_SELCHANGED:
		break;
	}
	return 0;
}

BOOL TIXAPI TixQueryDirectory(
	HWND hwndParent,			// [In] Parent window handle
	TCHAR *pszDirPath			// [In/out] Directory path buffer >= MAX_PATH
)
{
	HRESULT hResult;
	BROWSEINFO bi;
	BOOL bResult;
	LPITEMIDLIST pidlRoot = NULL, pidlBrowse = NULL;
	LPMALLOC pMalloc;
	TCHAR szDisplayName[MAX_PATH];

	// Get the Desktop PIDL to serve as root
	if (FAILED(hResult = SHGetSpecialFolderLocation(hwndParent,
		CSIDL_DESKTOP, &pidlRoot)))
		return FALSE;

	// Setup browse parameters
    bi.hwndOwner = hwndParent;
    bi.pidlRoot = pidlRoot;
    bi.pszDisplayName = szDisplayName;
    bi.lpszTitle = NULL;
    bi.ulFlags = BIF_RETURNONLYFSDIRS;	// Accept only real dirs
    bi.lpfn = __BffCallbackProc;
    bi.lParam = (LPARAM)pszDirPath;		// Must be plain string
    bi.iImage = 0;

    // Browse for a folder and return its PIDL, then convert to path
	bResult = FALSE;
    if ((pidlBrowse = SHBrowseForFolder(&bi)) != NULL)
        bResult = SHGetPathFromIDList(pidlBrowse, pszDirPath);

	// Get shell's IMalloc to free the PIDLs
	if (SUCCEEDED(hResult = SHGetMalloc(&pMalloc)))
	{
		(*pMalloc->lpVtbl->Free)(pMalloc, pidlRoot);
		(*pMalloc->lpVtbl->Free)(pMalloc, pidlBrowse);
		(*pMalloc->lpVtbl->Release)(pMalloc);
	}
	return bResult;
}

//---------------------------------------------------------------------------
// TixMakePath
//
// Concatenate a partial path and file name without buffer overflow.

BOOL TIXAPI TixMakePath(
	const TCHAR *pszPath,		// [In] Partial path
	const TCHAR *pszFileName,	// [In] File name to append
	TCHAR *pszResult,			// [Out] Output buffer
	UINT uResultCount			// [Out] Output buffer size in TCHARs
)
{
	TCHAR *pszPos, *pszEndPos;
#ifdef _MBCS
	TCHAR *pszPrevPos;
#endif

	if (pszResult == NULL || uResultCount == 0)
		return FALSE;

	// Calculate buffer end with room for \0 terminator
	pszEndPos = pszResult + uResultCount - 1;

	// Copy initial portion
	pszPos = TixStrCopy(pszPath, pszResult, pszEndPos);

	// Do we need to insert a dir separator?
#ifdef _MBCS
	if (pszPos > pszResult && pszPos < pszEndPos)
	{
		pszPrevPos = CharPrev(pszResult, pszPos);
		if (*pszPrevPos != _T('\\') && *pszPrevPos != _T('/') &&
			*pszPrevPos != _T(':'))
			*pszPos++ = _T('\\');
	}
#else
	if (pszPos > pszResult && pszPos < pszEndPos &&
		pszPos[-1] != _T('\\') && pszPos[-1] != _T('/') &&
		pszPos[-1] != _T(':'))
		*pszPos++ = _T('\\');
#endif

	// Append file name
	TixStrCopy(pszFileName, pszPos, pszEndPos);
	return TRUE;
}

//---------------------------------------------------------------------------
// TixGetFileName
//
// Get a pointer to the file name portion of a full or partial path.
//
// Returns a pointer to the start of the file name in the passed-in path, or
// to the start of the path if no directory components exist.

const TCHAR * TIXAPI TixGetFileName(
	const TCHAR *pszFilePath		// [In] Fully qualified file path
)
{
	const TCHAR *pszPos;
#ifdef _MBCS
	TCHAR *pszPrevPos;
#endif
	
	// Find the end of the path (the '\0')
	pszPos = pszFilePath + lstrlen(pszFilePath);
	
	// Scan backwards until start of string or a directory separator
#ifdef _MBCS
	while (pszPos > pszFilePath)
	{
		// Decrement first to detect dir seps at the start of the path
		pszPrevPos = CharPrev(pszFilePath, pszPos);
		if (*pszPrevPos == _T('\\') || *pszPrevPos == _T('/') ||
			*pszPrevPos == _T(':'))
			return pszPos;		// Might point at '\0' to indicate empty name
		pszPos = pszPrevPos;
	}
#else
	while (pszPos > pszFilePath)
	{
		// Decrement first to detect dir seps at the start of the path
		--pszPos;
		if (*pszPos == _T('\\') || *pszPos == _T('/') || *pszPos == _T(':'))
			return pszPos + 1;	// Might point at '\0' to indicate empty name
	}
#endif
	return pszFilePath;
}

//---------------------------------------------------------------------------
// TixPathRemoveRight
//
// Chop off the rightmost path component. Returns nonzero if the remaining
// path is not empty, or FALSE if it is empty.

BOOL TIXAPI TixPathRemoveRight(
	TCHAR *pszFilePath			// [In/out] Fully qualified file path
)
{
	TCHAR *pszPos;
#ifdef _MBCS
	TCHAR *pszPrevPos;
#endif
	
	// Find the end of the path (the '\0')
	pszPos = pszFilePath + lstrlen(pszFilePath);

	// If the path currently ends with a dir separator, ignore that.
#ifdef _MBCS
	if (pszPos > pszFilePath)
	{
		pszPrevPos = CharPrev(pszFilePath, pszPos);
		if (*pszPrevPos == _T('\\') || *pszPrevPos == _T('/'))
			pszPos = pszPrevPos;
	}
#else
	if (pszPos > pszFilePath &&
		(pszPos[-1] == _T('\\') || pszPos[-1] == _T('/')))
		--pszPos;
#endif

	// Scan backwards until start of string or a directory separator
#ifdef _MBCS
	while (pszPos > pszFilePath)
	{
		// Decrement first to detect dir seps at the start of the path
		pszPrevPos = CharPrev(pszFilePath, pszPos);
		if (*pszPrevPos == _T('\\') || *pszPrevPos == _T('/'))
		{
			pszPos = pszPrevPos;
			break;				// Terminate right here
		}
		else if (*pszPrevPos == _T(':'))
			break;				// Terminate after the ':'
		pszPos = pszPrevPos;
	}
#else
	while (pszPos > pszFilePath)
	{
		// Decrement first to detect dir seps at the start of the path
		--pszPos;
		if (*pszPos == _T('\\') || *pszPos == _T('/'))
			break;				// Terminate right here
		else if (*pszPos == _T(':'))
		{
			++pszPos;			// Terminate just after this one
			break;
		}
	}
#endif
	*pszPos = 0;				// Truncate the path
	return pszFilePath[0] != 0;
}

//---------------------------------------------------------------------------
// TixFileExists
//
// Test if a given file (or folder) exists.

BOOL TIXAPI TixFileExists(
	const TCHAR *pszFilePath		// [In] Fully qualified file path
)
{
	DWORD dwAttr = GetFileAttributes(pszFilePath);
	return dwAttr != INVALID_FILE_ATTRIBUTES;

/*	Alternative, if it must be a file and not a directory:
	return dwAttr != INVALID_FILE_ATTRIBUTES &&
		(dwAttr & FILE_ATTRIBUTE_DIRECTORY) == 0;
*/
}

//---------------------------------------------------------------------------
// Perform a recursive search for a key file (with optional wildcards). The
// search is pre-order: the parent folder is searched before any children.
// The function either returns the parent folder of the key file, or the
// full path to the file itself.
//
// NOTE: pszSearchPath may contain <symbolic> references.
//
// Returns nonzero if found, or FALSE (0) if not.

BOOL TIXAPI TixFindFile(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszSearchPath,	// [In] Top-level folder to start the search
	const TCHAR *pszKeyFile,	// [In] File spec to search for
	BOOL bParentFolder,			// [In] Return parent folder instead of file path
	TCHAR *pszResult,			// [Out] Path buffer for result, if found
	UINT uResultCount			// [In] Size of pszResult in TCHARs, in chars
)
{
	TCHAR szPath[MAX_PATH], szSearchSpec[MAX_PATH];
	WIN32_FIND_DATA wfd;
	HANDLE hFind;

	if (pTixRuntime == NULL)
		return FALSE;

	// Sort out the arguments
	TIXRT_ResolveSymbols(pTixRuntime, pszSearchPath, szPath,
		__countof(szPath));

	// Sanity checks to reduce error checking below
	if (szPath[0] == 0 || pszKeyFile == NULL || pszKeyFile[0] == 0 ||
		pszResult == NULL || uResultCount == 0)
		return FALSE;

	TIX_LOG((_T("SearchKeyPath: Searching %s for key file %s\r\n"),
		szPath, pszKeyFile));

	// Pass 1: search for matching file(s)/folder(s) in the initial folder.
	// We only need the first one.
	TixMakePath(szPath, pszKeyFile, szSearchSpec, __countof(szSearchSpec));
	hFind = FindFirstFile(szSearchSpec, &wfd);
	if (hFind != INVALID_HANDLE_VALUE)
	{
		// Found it -- return the containing folder path
		TIX_LOG((_T("TixFindFile: Found matching file/folder %s\r\n"),
			wfd.cFileName));
		if (bParentFolder)
			lstrcpyn(pszResult, szPath, uResultCount);
		else
			TixMakePath(szPath, wfd.cFileName, pszResult, uResultCount);

		FindClose(hFind);
		return TRUE;
	}

	// Pass 2: No matches in this folder; recurse into subfolders
	TixMakePath(szPath, _T("*.*"), szSearchSpec, __countof(szSearchSpec));
	hFind = FindFirstFile(szSearchSpec, &wfd);
	if (hFind != INVALID_HANDLE_VALUE)
	{
		do
		{
			// We want only directories, but not '.' or '..'
			if ((wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
				continue;
			if (wfd.cFileName[0] == _T('.') && (wfd.cFileName[1] == 0 ||
				(wfd.cFileName[1] == _T('.') && wfd.cFileName[2] == 0)))
				continue;

			// Recurse into this subdirectory
			TixMakePath(szPath, wfd.cFileName, szSearchSpec,
				__countof(szSearchSpec));
			if (TixFindFile(pTixRuntime, szSearchSpec, pszKeyFile,
				bParentFolder, pszResult, uResultCount))
			{
				// Found it
				FindClose(hFind);
				return TRUE;
			}
		}
		while (FindNextFile(hFind, &wfd));
		FindClose(hFind);
	}

	// If we get here, there were no matches
	return FALSE;
}

//---------------------------------------------------------------------------
// TixGetVersionInfo
//
// Get the fixed VERSIONINFO for a file, if it has any. Only executable
// files (.exe, .dll, .ocx, etc.) can have version info, but not all do.
//
// Returns nonzero if VERSIONINFO was found, or FALSE if not.

BOOL TIXAPI TixGetVersionInfo(
	const TCHAR *pszFilePath,	// [In] Fully qualified file path
	VS_FIXEDFILEINFO *pFFI		// [Out] Version info buffer
)
{
	HANDLE hHeap;
	DWORD dwVerSize;
	UINT uSize;
	void *pVerBlock;
	void *pSubBlock;
	BOOL bResult = FALSE;

	// Does the file have VERSIONINFO at all?
	if ((dwVerSize = GetFileVersionInfoSize((LPTSTR)pszFilePath, NULL)) == 0)
		return FALSE;

	// Allocate block for the data and load the data
	hHeap = GetProcessHeap();
	pVerBlock = HeapAlloc(hHeap, 0, dwVerSize);
	if (!GetFileVersionInfo((LPTSTR)pszFilePath, 0, dwVerSize, pVerBlock))
		goto lblCleanup;

	// Get the fixed info
	if (!VerQueryValue(pVerBlock, _T("\\"), &pSubBlock, &uSize) ||
		uSize < sizeof(VS_FIXEDFILEINFO))
		goto lblCleanup;

	// Copy the data
	TixMemMove(pFFI, pSubBlock, sizeof(VS_FIXEDFILEINFO));
	bResult = TRUE;

lblCleanup:
	HeapFree(hHeap, 0, pVerBlock);
	return bResult;
}

//---------------------------------------------------------------------------
// _TixParseDword
//
// Helper function: Parse a string to a DWORD value. Stops at the first
// non-digit. Returns the DWORD value and updates the string pointer
// in the process.

static DWORD TIXAPI _TixParseDword(const TCHAR **ppszStr)
{
	DWORD dwValue = 0;

	while (**ppszStr >= _T('0') && **ppszStr <= _T('9'))
	{
		dwValue *= 10;
		dwValue += **ppszStr - '0';
		++(*ppszStr);			// No MBCS issues with digits
	}
	return dwValue;
}

//---------------------------------------------------------------------------
// TixParseVersion
//
// Parse a string of the form a.b.c.d to a version-compatible number.
// The a.b part is returned as (65536 * a) + b in the high-order DWORD;
// the c.d is returned as (65536 * c) + d in the low-order DWORD. This is
// compatible with the usual VERSIONINFO conventions.

BOOL TIXAPI TixParseVersion(
	const TCHAR *pszVersion,	// [In] Version string as a.b.c.d
	DWORD *pdwVersionLow,		// [Out] Pointer to low-order version DWORD
	DWORD *pdwVersionHigh		// [Out] Pointer to high-order version DWORD
)
{
	if (pszVersion == NULL || pdwVersionLow == NULL || pdwVersionHigh == NULL)
		return FALSE;

	// Skip leading white space, if any
	while (*pszVersion == _T(' ') || *pszVersion == _T('\t'))
		++pszVersion;			// No MBCS issues with these

	// High-order DWORD
	*pdwVersionHigh = LOWORD(_TixParseDword(&pszVersion)) << 16;
	if (*pszVersion == _T('.'))
	{
		++pszVersion;			// No MBCS issues with these
		*pdwVersionHigh += LOWORD(_TixParseDword(&pszVersion));
	}

	// Low-order DWORD
	*pdwVersionLow = 0;
	if (*pszVersion == _T('.'))
	{
		++pszVersion;			// No MBCS issues with these
		*pdwVersionLow = LOWORD(_TixParseDword(&pszVersion)) << 16;
	}
	if (*pszVersion == _T('.'))
	{
		++pszVersion;			// No MBCS issues with these
		*pdwVersionLow += LOWORD(_TixParseDword(&pszVersion));
	}
	return TRUE;
}

//---------------------------------------------------------------------------
// TixFormatVersion
//
// Format a version number to the form a.b.c.d as per TixParseVersion()
// conventions.

BOOL TIXAPI TixFormatVersion(
	DWORD dwVersionLow,			// [In] Low-order version DWORD
	DWORD dwVersionHigh,		// [In] High-order version DWORD
	TCHAR *pszBuffer,			// [Out] Output buffer
	UINT uResultCount			// [In] Output buffer count (>= 24 TCHARs)
)
{
	if (pszBuffer == NULL || uResultCount < 24)
		return FALSE;

	// The following is at most 4 * 5 + 3 + 1 characters long
	wsprintf(pszBuffer, _T("%u.%u.%u.%u"),
		HIWORD(dwVersionHigh), LOWORD(dwVersionHigh),
		HIWORD(dwVersionLow), LOWORD(dwVersionLow));

	return TRUE;
}

//---------------------------------------------------------------------------
// TixCopyDirectory
//
// Copy the contents of one directory to another. Returns 0 if successful,
// or the Win32 error code if an error occurred.

DWORD TIXAPI TixCopyDirectory(
	const TCHAR *pszSourceDir,	// [In] Source directory
	const TCHAR *pszTargetDir,	// [In] Target directory
	BOOL bOverwriteExisting		// [In] TRUE => overwrite existing files
)
{
	HANDLE hFind;
	WIN32_FIND_DATA wfd;
	DWORD dwResult, dwAttributes;
	TCHAR szSourceBuf[MAX_PATH], szTargetBuf[MAX_PATH];

	TIX_LOG((_T("Copying directory %s to %s...\r\n"),
		pszSourceDir, pszTargetDir));

	// Create the target directory if necessary. Ignore result; the function
	// fails if the directory already exists.
	CreateDirectory(pszTargetDir, NULL);

	// Build the source search mask and start the file enumeration
	TixMakePath(pszSourceDir, _T("*.*"), szSourceBuf,
		__countof(szSourceBuf));
	hFind = FindFirstFile(szSourceBuf, &wfd);
	if (hFind == INVALID_HANDLE_VALUE)
	{
		TIX_LOG((_T("%s: no match\r\n"), szSourceBuf));
		return GetLastError();
	}

	dwResult = 0;
	do
	{
		// Skip directories
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			continue;

		// Build source and destination paths
		TixMakePath(pszSourceDir, wfd.cFileName, szSourceBuf,
			__countof(szSourceBuf));
		TixMakePath(pszTargetDir, wfd.cFileName, szTargetBuf,
			__countof(szTargetBuf));

		// Check if we must/may overwrite
		dwAttributes = GetFileAttributes(szTargetBuf);
		if (dwAttributes != INVALID_FILE_ATTRIBUTES)
		{
			if (!bOverwriteExisting)
			{
				TIX_LOG((_T("Skipping %s to %s...\r\n"), szSourceBuf,
					szTargetBuf));
				continue;		// Don't clobber existing file
			}

			// Reset file protection attributes
			SetFileAttributes(szTargetBuf, dwAttributes & ~TIX_ATTR_PROTECTED);
		}

		// Copy the file
		TIX_LOG((_T("Copying %s to %s...\r\n"), szSourceBuf, szTargetBuf));
		if (!CopyFile(szSourceBuf, szTargetBuf, FALSE))
			dwResult = GetLastError();	// Set error, but keep copying
	}
	while (FindNextFile(hFind, &wfd));

	FindClose(hFind);
	return dwResult;
}

//---------------------------------------------------------------------------
// TixDeleteFiles
//
// Delete all files that match the given wildcard mask. Returns 0 if
// successful, or the Win32 error code if an error occurred.

DWORD TIXAPI TixDeleteFiles(
	const TCHAR *pszDir,			// [In] Directory to delete
	const TCHAR *pszFileMask		// [In] Wildcard mask of files to delete
)
{
	HANDLE hFind;
	WIN32_FIND_DATA wfd;
	DWORD dwResult;
	TCHAR szPathBuf[MAX_PATH];

	// Build the file search mask and start the file enumeration
	TixMakePath(pszDir, pszFileMask, szPathBuf, __countof(szPathBuf));
	hFind = FindFirstFile(szPathBuf, &wfd);
	if (hFind == INVALID_HANDLE_VALUE)
		return GetLastError();

	dwResult = 0;
	do
	{
		// Skip directories
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			continue;

		// Build file path
		TixMakePath(pszDir, wfd.cFileName, szPathBuf, __countof(szPathBuf));

		// Reset file protection attributes, if any, then delete
		SetFileAttributes(szPathBuf, wfd.dwFileAttributes & ~TIX_ATTR_PROTECTED);
		TIX_LOG((_T("Deleting %s...\r\n"), szPathBuf));
		if (!DeleteFile(szPathBuf))
			dwResult = GetLastError();	// Set error, but keep going
	}
	while (FindNextFile(hFind, &wfd));

	FindClose(hFind);
	return dwResult;
}

//---------------------------------------------------------------------------
// TixDeleteDirectory
//
// Delete the contents of a directory, then delete the directory itself.
// Returns 0 if successful, or the Win32 error code if an error occurred.

DWORD TIXAPI TixDeleteDirectory(
	const TCHAR *pszDir			// [In] Directory to delete
)
{
	DWORD dwResult;

	TIX_LOG((_T("Deleting folder %s...\r\n"), pszDir));

	// Delete the files in the directory
	dwResult = TixDeleteFiles(pszDir, _T("*.*"));

	// Delete the directory itself
	if (!RemoveDirectory(pszDir))
		dwResult = GetLastError();

	return dwResult;
}

//---------------------------------------------------------------------------
// TixIsProcessRunning
//
// Check if a particular process is running. Only the process name is used
// for the test. This check can only be performed on NT4 if it happens to
// have the PSAPI DLL installed.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

BOOL TIXAPI TixIsProcessRunning(
	const TCHAR *pszProcessName	// [In] Process name to check for
)
{
	return TixIsProcessRunningNT(pszProcessName, NULL, 0) ||
		TixIsProcessRunningEx(pszProcessName, NULL, 0);
}

//---------------------------------------------------------------------------
// TixIsProcessRunningEx
//
// Check if a particular process is running. Only the process name is used
// for the test. This extended version accepts a callback function that is
// invoked for each instance of the process that is running.
//
// NOTE: Process name should be <= 15 characters on W2K and later. Does not
// work at all on NT4 and always resturns FALSE on that platform.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

#include <tlhelp32.h>

#ifdef _UNICODE
#define _tPROCESSENTRY32 PROCESSENTRY32W
#else
#define _tPROCESSENTRY32 PROCESSENTRY32
#endif

typedef HANDLE (WINAPI *fCreateToolhelp32Snapshot)(DWORD dwFlags,
	DWORD th32ProcessID);
typedef BOOL (WINAPI *fProcess32First)(HANDLE hSnapshot,
	_tPROCESSENTRY32 *lppe);
typedef BOOL (WINAPI *fProcess32Next)(HANDLE hSnapshot,
	_tPROCESSENTRY32 *lppe);

BOOL TIXAPI TixIsProcessRunningEx(
	const TCHAR *pszProcessName,// [In] Process name to look for
	fTixProcCB pCallback,		// [In] Callback function
	LPARAM lCookie				// [In] User-defined cookie
)
{
	fCreateToolhelp32Snapshot pCreateToolhelp32Snapshot;
	fProcess32First pProcess32First;
	fProcess32Next pProcess32Next;
	HMODULE hKernel;
	HANDLE hSnapshot;
	_tPROCESSENTRY32 pe32;
	__cTixProcInfo pi;
	BOOL bRunning, bOK;

	// TOOLHELP32 functions -- Win9x, and Win2000 and later
	//
	// Note: Contrary to SDK docs, Win2000 only returns file names in
	// PROCESSENTRY32; Win9x does return the full path. Worse still,
	// Win2000 & later truncate the file name to 15 characters + \0,
	// presumably because of internal implementation issues.
	//
	// Note: 8.3 issues are ignored.

	// Grab the various entry points
	hKernel = GetModuleHandleA("kernel32");
	pCreateToolhelp32Snapshot = (fCreateToolhelp32Snapshot)
		GetProcAddress(hKernel, "CreateToolhelp32Snapshot");
	pProcess32First = (fProcess32First)GetProcAddress(hKernel,
		"Process32First");
	pProcess32Next = (fProcess32Next)GetProcAddress(hKernel,
		"Process32Next");

	if (pCreateToolhelp32Snapshot == NULL || pProcess32First == NULL ||
		pProcess32Next == NULL)
	{
		TIX_LOG((_T("ToolHelp32 entry points not available -- skipping test\r\n")));
		return FALSE;
	}

	// Take a process snapshot
	hSnapshot = pCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if (hSnapshot == NULL)
	{
		TIX_LOG((_T("Cannot create snapshot -- skipping test\r\n")));
		return FALSE;
	}

	// Walk the processes in the snapshot
	pszProcessName = TixGetFileName(pszProcessName);	// Just in case
	pe32.dwSize = sizeof(pe32);
	bRunning = FALSE;

	for (bOK = pProcess32First(hSnapshot, &pe32); bOK;
		bOK = pProcess32Next(hSnapshot, &pe32))
	{
		TIX_LOG((_T("Next process: %s\r\n"), pe32.szExeFile));
		
		// Only compare file names - W2K and later don't provide more
		if (lstrcmpi(TixGetFileName(pe32.szExeFile), pszProcessName) == 0)
		{
			TIX_LOG((_T("*** Match found on %s ***\r\n"), pszProcessName));
			bRunning = TRUE;
			if (pCallback == NULL) break;

			// Give callback a chance to process
			pi.dwProcessID = pe32.th32ProcessID;
			pi.hProcess = OpenProcess(
				PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
				FALSE, pi.dwProcessID);
			lstrcpyn(pi.szExePath, pe32.szExeFile, __countof(pi.szExePath));

			if (!(*pCallback)(pszProcessName, &pi, lCookie))
			{
				CloseHandle(pi.hProcess);
				break;
			}
			CloseHandle(pi.hProcess);
		}
	}
	CloseHandle(hSnapshot);
	return bRunning;
}

//---------------------------------------------------------------------------
// TixIsProcessRunningNT
//
// Check if a particular process is running. Only the process name is used
// for the test. This version only runs on NT-based platforms.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

typedef BOOL (WINAPI *fEnumProcesses)(DWORD *lpidProcess, DWORD cb,
	DWORD *cbNeeded);

typedef DWORD (WINAPI *fGetModuleFileNameEx)(HANDLE hProcess,
	HMODULE hModule, LPTSTR lpFilename, DWORD nSize);

BOOL TIXAPI TixIsProcessRunningNT(
	const TCHAR *pszProcessName,// [In] Process name to look for
	fTixProcCB pCallback,		// [In] Callback function
	LPARAM lCookie				// [In] User-defined cookie
)
{
	fEnumProcesses pEnumProcesses;
	fGetModuleFileNameEx pGetModuleFileNameEx;
	HMODULE hPSAPI;
	HANDLE hHeap;
	DWORD *pdwProcessIDs, dwProcSize, dwProcCount, i;
	void *pTemp;
	__cTixProcInfo pi;
	TCHAR *pszFileName;
	BOOL bRunning;

	// PSAPI functions -- NT4 and later, but on NT4 it's an optional module
	// and not installed by default.
	//
	// Note: 8.3 issues are ignored.

	hPSAPI = LoadLibraryA("psapi.dll");
	if (hPSAPI == NULL)
	{
		TIX_LOG((_T("PSAPI DLL not available -- skipping test\r\n")));
		return FALSE;
	}

	// Grab the various entry points
	pEnumProcesses = (fEnumProcesses)
		GetProcAddress(hPSAPI, "EnumProcesses");
#ifdef _UNICODE
	pGetModuleFileNameEx = (fGetModuleFileNameEx)
		GetProcAddress(hPSAPI, "GetModuleFileNameExW");
#else
	pGetModuleFileNameEx = (fGetModuleFileNameEx)
		GetProcAddress(hPSAPI, "GetModuleFileNameExA");
#endif

	if (pEnumProcesses == NULL || pGetModuleFileNameEx == NULL)
	{
		TIX_LOG((_T("PSAPI entry points not available -- skipping test\r\n")));
		FreeLibrary(hPSAPI);
		return FALSE;
	}

	// Allocate a process ID buffer and keep asking for an enumeration
	// until we are to have enough ID slots.
	hHeap = GetProcessHeap();
	dwProcSize = 1024 * sizeof(DWORD);
	dwProcCount = 0;
	pdwProcessIDs = HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwProcSize);

	while ((*pEnumProcesses)(pdwProcessIDs, dwProcSize, &dwProcCount) &&
		dwProcCount >= dwProcSize)
	{
		// Expand the process ID array
		pTemp = HeapReAlloc(hHeap, HEAP_ZERO_MEMORY, pdwProcessIDs,
			2 * dwProcSize);
		if (pTemp == NULL) break;

		pdwProcessIDs = pTemp;
		dwProcSize *= 2;
		dwProcCount = 0;
	}

	// Walk the available processes
	dwProcCount /= sizeof(DWORD);
	pszProcessName = TixGetFileName(pszProcessName);	// Just in case
	bRunning = FALSE;

	for (i = 0; i < dwProcCount; ++i)
	{
		// Try to open the process handle. PID 0 (System Idle) results in
		// ERROR_INVALID_PARAMETER, so skip that one.
		if (pdwProcessIDs[i] == 0) continue;
		pi.dwProcessID = pdwProcessIDs[i];
		pi.hProcess = OpenProcess(
			PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
			FALSE, pi.dwProcessID);
		if (pi.hProcess == NULL)
		{
			TIX_LOG((_T("Process %u: Error %u while opening handle\r\n"),
				pi.dwProcessID, GetLastError()));
			continue;
		}

		// Get the .exe module's path
		if ((*pGetModuleFileNameEx)(pi.hProcess, NULL, pi.szExePath,
			__countof(pi.szExePath)) > 0)
		{
			TIX_LOG((_T("Next process.exe: %s\r\n"), pi.szExePath));

			pszFileName = (TCHAR *)TixGetFileName(pi.szExePath);
			if (lstrcmpi(pszFileName, pszProcessName) == 0)
			{
				TIX_LOG((_T("*** Match found on %s ***\r\n"), pszProcessName));
				bRunning = TRUE;

				// Give callback a chance to process
				if (pCallback == NULL ||
					!(*pCallback)(pszProcessName, &pi, lCookie))
				{
					CloseHandle(pi.hProcess);
					break;
				}
			}
		}
		else
		{
			TIX_LOG((_T("Process %u: Error %u while querying process path\r\n"),
				pi.dwProcessID, GetLastError()));
		}
		CloseHandle(pi.hProcess);
	}

	// Clean up
	HeapFree(hHeap, 0, pdwProcessIDs);
	FreeLibrary(hPSAPI);
	return bRunning;
}

//---------------------------------------------------------------------------
// Terminate a shell link object. This releases all COM interfaces.
// Returns nonzero on success, or FALSE on failure.

typedef struct cTixShellLink
{
	IShellLink *pLink;					// Main interface pointer
	IPersistFile *pFile;				// Persistence interface
} cTixShellLink;

static BOOL TIXAPI TixShellLink_Term(
	cTixShellLink *pShellLink	// Shell link object to terminate
)
{
	if (pShellLink->pLink != NULL)
		(*pShellLink->pLink->lpVtbl->Release)(pShellLink->pLink);
	if (pShellLink->pFile != NULL)
		(*pShellLink->pFile->lpVtbl->Release)(pShellLink->pFile);

	// Bring object in a safe state
	pShellLink->pLink = NULL;
	pShellLink->pFile = NULL;
	return TRUE;
}

//---------------------------------------------------------------------------
// Initialize a shell link object. This obtains all COM interfaces. The
// object must be cleaned up by a call to TixShellLink_Term().
// Returns nonzero on success, or FALSE on failure.

static BOOL TIXAPI TixShellLink_Init(
	cTixShellLink *pShellLink	// Shell link object to initialize
)
{
	// Bring object in a safe state
	pShellLink->pLink = NULL;
	pShellLink->pFile = NULL;

	// Create instance of IShellLink
	if (FAILED(CoCreateInstance(&CLSID_ShellLink, 0,
		CLSCTX_INPROC_SERVER, &IID_IShellLink, (void **)&pShellLink->pLink)))
		return FALSE;

	// Get its IPersistFile interface pointer
	if (FAILED((*pShellLink->pLink->lpVtbl->QueryInterface)(
		pShellLink->pLink, &IID_IPersistFile, (void **)&pShellLink->pFile)))
	{
		TixShellLink_Term(pShellLink);
		return FALSE;
	}
	return TRUE;
}

//---------------------------------------------------------------------------
// TixCreateShortcut
//
// Create a shortcut (shell link) to a file or folder. The shortcut will
// only be created if the target file or folder exists.
//
// NOTE: All string parameters may contain <symbolic> references; they
// will be resolved automatically. Use the following expressions for the
// standard shortcut locations in pszFolderPath:
//
//	<Desktop>		- the Windows Desktop
//	<StartMenu>		- the Windows Start menu
//	<Programs>		- the Windows Start > Programs menu
//	<ProgGroupDir>	- the application's Program Group folder
//
// Returns nonzero if successful, or FALSE (0) if failed.

BOOL TIXAPI TixCreateShortcut(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszFolderPath,	// [In] Folder for the shortcut
	const TCHAR *pszTitle,		// [In] Shortcut title
	const TCHAR *pszDescription,// [In,opt] Shortcut description
	const TCHAR *pszTargetPath,	// [In] Target path for shortcut
	const TCHAR *pszArguments,	// [In,opt] Arguments for target
	const TCHAR *pszWorkDir,	// [In,opt] Working directory
	const TCHAR *pszIconPath,	// [In,opt] Icon path for shortcut
	int nIconIndex,				// [In] Icon index for shortcut
	int nShowCommand			// [In] WM_SHOW command type
)
{
	TCHAR szFolder[MAX_PATH], szBuffer[1024], szPathBuf[MAX_PATH + 32];
#ifndef _UNICODE
	WCHAR szPathBufW[MAX_PATH];
#endif
	cTixShellLink link;
	BOOL bResult;

	// Check if the shortcut target and installation folder exist at all
	TIXRT_ResolveSymbols(pTixRuntime, pszFolderPath, szFolder,
		__countof(szFolder));
	TIXRT_ResolveSymbols(pTixRuntime, pszTargetPath, szBuffer,
		__countof(szBuffer));
	if (!TixFileExists(szFolder) || !TixFileExists(szBuffer))
		return FALSE;

	// Initialize the shell link object
	if (!TixShellLink_Init(&link))
		return FALSE;

	// Set up the shell link object
	(*link.pLink->lpVtbl->SetPath)(link.pLink, szBuffer);
	(*link.pLink->lpVtbl->SetShowCmd)(link.pLink, nShowCommand);

	// Sort out the other arguments
	TIXRT_ResolveSymbols(pTixRuntime, pszArguments, szBuffer,
		__countof(szBuffer));
	if (szBuffer[0] != 0)
		(*link.pLink->lpVtbl->SetArguments)(link.pLink, szBuffer);

	TIXRT_ResolveSymbols(pTixRuntime, pszWorkDir, szBuffer,
		__countof(szBuffer));
	if (szBuffer[0] != 0)
		(*link.pLink->lpVtbl->SetWorkingDirectory)(link.pLink, szBuffer);

	TIXRT_ResolveSymbols(pTixRuntime, pszDescription, szBuffer,
		__countof(szBuffer));
	if (szBuffer[0] != 0)
		(*link.pLink->lpVtbl->SetDescription)(link.pLink, szBuffer);

	TIXRT_ResolveSymbols(pTixRuntime, pszIconPath, szBuffer,
		__countof(szBuffer));
	if (szBuffer[0] != 0)
		(*link.pLink->lpVtbl->SetIconLocation)(link.pLink, szBuffer, nIconIndex);

	// Finally, save the shortcut to a disk file
	TIXRT_ResolveSymbols(pTixRuntime, pszTitle, szBuffer,
		__countof(szBuffer));
	TixMakePath(szFolder, szBuffer, szPathBuf, MAX_PATH /*__countof(szPathBuf)*/);
	lstrcat(szPathBuf, _T(".lnk"));

#ifdef _UNICODE
	bResult = SUCCEEDED((*link.pFile->lpVtbl->Save)(link.pFile, szPathBuf, TRUE));
#else
	MultiByteToWideChar(CP_ACP, 0, szPathBuf, -1, szPathBufW,
		__countof(szPathBufW));
	bResult = SUCCEEDED((*link.pFile->lpVtbl->Save)(link.pFile, szPathBufW, TRUE));
#endif

	// Clean up
	TixShellLink_Term(&link);
	return bResult;
}

//---------------------------------------------------------------------------
// TixDeleteShortcut
//
// Deletes a shortcut (shell link) to a file or folder.
//
// NOTE: All string parameters may contain <symbolic> references; they
// will be resolved automatically.
//
// Returns nonzero if successful, or FALSE (0) if failed.

BOOL TIXAPI TixDeleteShortcut(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszFolderPath,	// [In] Folder for the shortcut
	const TCHAR *pszTitle		// [In] Shortcut title
)
{
	TCHAR szFolder[MAX_PATH], szTitle[MAX_PATH], szLinkPath[MAX_PATH];

	// Sort out the arguments
	TIXRT_ResolveSymbols(pTixRuntime, pszFolderPath, szFolder,
		__countof(szFolder));
	TIXRT_ResolveSymbols(pTixRuntime, pszTitle, szTitle, __countof(szTitle));

	// Generate the shortcut path
	TixMakePath(szFolder, szTitle, szLinkPath, __countof(szLinkPath));
	lstrcat(szLinkPath, _T(".lnk"));

	return DeleteFile(szLinkPath);
}
