/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixUtils.h 4     10-12-05 12:16 Ron $

	Copyright (c) 2004 Tarma Software Research Ltd. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	03-09-2004 20:38

	Declaration of helper functions for Tarma extension DLLs.
****************************************************************************/

#pragma once

#ifndef __TIXDEFS_H
#include "TixDefs.h"	// Base declarations
#endif

#ifdef __cplusplus
extern "C" {
#endif

//===========================================================================
// Implementation helpers
//
// Various helper functions implemented locally to prevent pulling in the
// C/C++ runtime libs and keep the DLL size as small as possible.
//===========================================================================

#ifndef INVALID_FILE_ATTRIBUTES
#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
#endif

// Extension DLL module handle for resource extraction etc.
extern HINSTANCE g_hTixDll;

//---------------------------------------------------------------------------
// Tracing functionality; only present if TIX_TRACE is defined

// Define TIX_TRACE to let the DLL create a trace of all handler calls.
// The trace log is written to the TEMP directory as _Tix.log
#ifdef _DEBUG
#define TIX_TRACE		// TODO: #define to enable diagnostic tracing
#endif

#ifdef TIX_TRACE

#define TIX_OPENLOG()	_TixTraceOpenLog()
#define TIX_CLOSELOG()	_TixTraceCloseLog()
#define TIX_LOG(x)		_TixTraceLog x
#define TIX_MSGNAME(id)	_TixGetMessageName(id)

BOOL TIXAPI _TixTraceOpenLog(void);
void TIXAPI _TixTraceCloseLog(void);
BOOL TIXAPI _TixTraceLog(const TCHAR *pszFormat, ...);

const TCHAR * TIXAPI _TixGetMessageName(DWORD dwMessageID);

#else	// Not TIX_TRACE

#define TIX_OPENLOG()	((void)0)
#define TIX_CLOSELOG()	((void)0)
#define TIX_LOG(x)		((void)0)
#define TIX_MSGNAME(id)	#id

#endif	// TIX_TRACE

//---------------------------------------------------------------------------
// TixMemMove
//
// Copy one block of memory to another; handles overlap.

void TIXAPI TixMemMove(void *pDest, const void *pSrc, SIZE_T uCount);

//---------------------------------------------------------------------------
// TixStrCopy
//
// Safe string copy function. Returns a pointer to the end of the
// destination string, i.e., to its \0 character.

TCHAR * TIXAPI TixStrCopy(const TCHAR *pszSrc, TCHAR *pszDest,
	TCHAR *pszDestEnd);

//---------------------------------------------------------------------------
// TixMessageBox
//
// Display a message box. Can be called in all situations; will only
// display if not in silent mode.
//
// Returns the message box result; in silent mode, returns nDefaultResult.

int __cdecl TixMessageBox(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	UINT uMBFlags,				// [In] Message box flags as per MessageBox()
	int nDefaultResult,			// [In] Default result for silent mode
	LPCTSTR lpFormat,			// [In] wsprintf-style format string
	...							// [In] Optional format args
);

//---------------------------------------------------------------------------
// TixLoadStringEx
//
// Load a language-specific string resource. The usual FindResourceEx()
// does not work as-is, because individual strings are stored in string
// tables, and only the string tables can be found directly.
//
// Returns nonzero if the string was found (even if truncated), FALSE if not.

BOOL TIXAPI TixLoadStringEx(
	HINSTANCE hModule,			// [In] Resource module handle
	UINT uStringID,				// [In] String resource ID
	WORD wLanguageID,			// [In] Language ID as per MAKELANGID()
	TCHAR *pszResult,			// [Out] Output buffer
	UINT uResultCount			// [In] Size of pszResult in TCHARs
);

//---------------------------------------------------------------------------
// TixDialogBox
//
// Run a modal dialog box. This is similar to DialogBoxParam(), but adds
// language-dependent dialog template look-up.
//
// Returns the result code from the dialog's message loop, or -1 on error.

int TIXAPI TixDialogBox(
	HINSTANCE hModule,			// [In] Module containing the dialog resource
	LPCTSTR lpTemplateName,		// [In] Dialog template name/ID
	WORD wLanguageID,			// [In] Language ID as per MAKELANGID()
	HWND hwndParent,			// [In] Parent window handle
	DLGPROC lpDialogFunc,		// [In] Dialog function
	LPARAM lParam				// [In] Dialog initialization parameter
);

//---------------------------------------------------------------------------
// TixQueryDirectory
//
// Query the user for a folder path.

BOOL TIXAPI TixQueryDirectory(
	HWND hwndParent,			// [In] Parent window handle
	TCHAR *pszDirPath			// [In/out] Directory path buffer >= MAX_PATH
);

//---------------------------------------------------------------------------
// TixMakePath
//
// Concatenate a partial path and file name without buffer overflow.

BOOL TIXAPI TixMakePath(
	const TCHAR *pszPath,		// [In] Partial path
	const TCHAR *pszFileName,	// [In] File name to append
	TCHAR *pszResult,			// [Out] Output buffer
	UINT uResultCount			// [In] Size of pszResult in TCHARs
);

//---------------------------------------------------------------------------
// TixGetFileName
//
// Get a pointer to the file name portion of a full or partial path.
//
// Returns a pointer to the start of the file name in the passed-in path, or
// to the start of the path if no directory components exist.

const TCHAR * TIXAPI TixGetFileName(
	const TCHAR *pszFilePath		// [In] Fully qualified file path
);

//---------------------------------------------------------------------------
// TixPathRemoveRight
//
// Chop off the rightmost path component. Returns nonzero if the remaining
// path is not empty, or FALSE if it is empty.

BOOL TIXAPI TixPathRemoveRight(
	TCHAR *pszFilePath			// [In/out] Fully qualified file path
);

//---------------------------------------------------------------------------
// TixFileExists
//
// Test if a given file (or folder) exists.

BOOL TIXAPI TixFileExists(
	const TCHAR *pszFilePath		// [In] Fully qualified file path
);

//---------------------------------------------------------------------------
// Perform a recursive search for a key file (with optional wildcards). The
// search is pre-order: the parent folder is searched before any children.
// The function either returns the parent folder of the key file, or the
// full path to the file itself.
//
// NOTE: pszSearchPath may contain <symbolic> references.
//
// Returns nonzero if found, or FALSE (0) if not.

BOOL TIXAPI TixFindFile(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszSearchPath,	// [In] Top-level folder to start the search
	const TCHAR *pszKeyFile,	// [In] File spec to search for
	BOOL bParentFolder,			// [In] Return parent folder instead of file path
	TCHAR *pszResult,			// [Out] Path buffer for result, if found
	UINT uResultCount			// [In] Size of pszResult in TCHARs
);

//---------------------------------------------------------------------------
// TixGetVersionInfo
//
// Get the fixed VERSIONINFO for a file, if it has any. Only executable
// files (.exe, .dll, .ocx, etc.) can have version info, but not all do.
//
// Returns nonzero if VERSIONINFO was found, or FALSE if not.

BOOL TIXAPI TixGetVersionInfo(
	const TCHAR *pszFilePath,	// [In] Fully qualified file path
	VS_FIXEDFILEINFO *pFFI		// [Out] Version info buffer
);

//---------------------------------------------------------------------------
// TixParseVersion
//
// Parse a string of the form a.b.c.d to a version-compatible number.
// The a.b part is returned as (65536 * a) + b in the high-order DWORD;
// the c.d is returned as (65536 * c) + d in the low-order DWORD. This is
// compatible with the usual VERSIONINFO conventions.

BOOL TIXAPI TixParseVersion(
	const TCHAR *pszVersion,	// [In] Version string as a.b.c.d
	DWORD *pdwVersionLow,		// [Out] Pointer to low-order version DWORD
	DWORD *pdwVersionHigh		// [Out] Pointer to high-order version DWORD
);

//---------------------------------------------------------------------------
// TixFormatVersion
//
// Format a version number to the form a.b.c.d as per TixParseVersion()
// conventions.

BOOL TIXAPI TixFormatVersion(
	DWORD dwVersionLow,			// [In] Low-order version DWORD
	DWORD dwVersionHigh,		// [In] High-order version DWORD
	TCHAR *pszResult,			// [Out] Output buffer
	UINT uResultCount			// [In] Size of pszResult in TCHARs
);

//---------------------------------------------------------------------------
// TixCopyDirectory
//
// Copy the contents of one directory to another. Returns 0 if successful,
// or the Win32 error code if an error occurred.

DWORD TIXAPI TixCopyDirectory(
	const TCHAR *pszSourceDir,	// [In] Source directory
	const TCHAR *pszTargetDir,	// [In] Target directory
	BOOL bOverwriteExisting		// [In] TRUE => overwrite existing files
);

//---------------------------------------------------------------------------
// TixDeleteFiles
//
// Delete all files that match the given wildcard mask. Returns 0 if
// successful, or the Win32 error code if an error occurred.

DWORD TIXAPI TixDeleteFiles(
	const TCHAR *pszDir,		// [In] Directory to delete
	const TCHAR *pszFileMask	// [In] Wildcard mask of files to delete
);

//---------------------------------------------------------------------------
// TixDeleteDirectory
//
// Delete the contents of a directory, then delete the directory itself.
// Returns 0 if successful, or the Win32 error code if an error occurred.

DWORD TIXAPI TixDeleteDirectory(
	const TCHAR *pszDir			// [In] Directory to delete
);

//---------------------------------------------------------------------------
// TixIsProcessRunning
//
// Check if a particular process is running. Only the process name is used
// for the test. This check can only be performed on NT4 if it happens to
// have the PSAPI DLL installed.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

BOOL TIXAPI TixIsProcessRunning(
	const TCHAR *pszProcessName	// [In] Process name to check for
);

//---------------------------------------------------------------------------
// TixIsProcessRunningEx
//
// Check if a particular process is running. Only the process name is used
// for the test. This extended version accepts a callback function that is
// invoked for each instance of the process that is running.
//
// NOTE: Process name should be <= 15 characters on W2K and later. Does not
// work at all on NT4 and always resturns FALSE on that platform.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

typedef struct __cTixProcInfoA
{
	HANDLE hProcess;			// Open for query only
	DWORD dwProcessID;			// Process ID
	ACHAR szExePath[MAX_PATH];	// May be only file name
} __cTixProcInfoA;

typedef struct __cTixProcInfoW
{
	HANDLE hProcess;			// Open for query only
	DWORD dwProcessID;			// Process ID
	WCHAR szExePath[MAX_PATH];	// May be only file name
} __cTixProcInfoW;

#ifdef _UNICODE
#define __cTixProcInfo __cTixProcInfoW
#else
#define __cTixProcInfo __cTixProcInfoA
#endif

// Process enumeration callback; must return TRUE to continue looking for
// further instances of the same process, or FALSE to stop enumerating.
typedef BOOL (TIXAPI *fTixProcCB)(
	const TCHAR *pszProcessName,// [In] Original process name
	const __cTixProcInfo *pProc,// [In] Running process info
	LPARAM lCookie				// [In] User-defined cookie
);

BOOL TIXAPI TixIsProcessRunningEx(
	const TCHAR *pszProcessName,// [In] Process name to look for
	fTixProcCB pCallback,		// [In] Callback function
	LPARAM lCookie				// [In] User-defined cookie
);

//---------------------------------------------------------------------------
// TixIsProcessRunningNT
//
// Check if a particular process is running. Only the process name is used
// for the test. This version only runs on NT-based platforms.
//
// Returns nonzero if it is, or FALSE if not or if the check could not be
// performed.

BOOL TIXAPI TixIsProcessRunningNT(
	const TCHAR *pszProcessName,// [In] Process name to look for
	fTixProcCB pCallback,		// [In] Callback function
	LPARAM lCookie				// [In] User-defined cookie
);

//---------------------------------------------------------------------------
// TixCreateShortcut
//
// Create a shortcut (shell link) to a file or folder. The shortcut will
// only be created if the target file or folder exists.
//
// NOTE: All string parameters may contain <symbolic> references; they
// will be resolved automatically. Use the following expressions for the
// standard shortcut locations in pszFolderPath:
//
//	<Desktop>		- the Windows Desktop
//	<StartMenu>		- the Windows Start menu
//	<Programs>		- the Windows Start > Programs menu
//	<ProgGroupDir>	- the application's Program Group folder
//
// Returns nonzero if successful, or FALSE (0) if failed.

BOOL TIXAPI TixCreateShortcut(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszFolderPath,	// [In] Folder for the shortcut
	const TCHAR *pszTitle,		// [In] Shortcut title
	const TCHAR *pszDescription,// [In,opt] Shortcut description
	const TCHAR *pszTargetPath,	// [In] Target path for shortcut
	const TCHAR *pszArguments,	// [In,opt] Arguments for target
	const TCHAR *pszWorkDir,	// [In,opt] Working directory
	const TCHAR *pszIconPath,	// [In,opt] Icon path for shortcut
	int nIconIndex,				// [In] Icon index for shortcut
	int nShowCommand			// [In] WM_SHOW command type
);

//---------------------------------------------------------------------------
// TixDeleteShortcut
//
// Deletes a shortcut (shell link) to a file or folder.
//
// NOTE: All string parameters may contain <symbolic> references; they
// will be resolved automatically.
//
// Returns nonzero if successful, or FALSE (0) if failed.

BOOL TIXAPI TixDeleteShortcut(
	iTixRuntime *pTixRuntime,	// [In] TIX runtime engine
	const TCHAR *pszFolderPath,	// [In] Folder for the shortcut
	const TCHAR *pszTitle		// [In] Shortcut title
);


#ifdef __cplusplus
}
#endif
