// $Header: /Dev/Tin3/InstallDir/WebUpdate/Samples/WebUpdate/WebUpdate.cpp 3     31/08/06 15:55 Dave $
//
// Copyright (c) 1990-2006 Tarma Software Research. All rights reserved.
//
// Project:	Tarma Installer
// Author:	Ron van der Wal
// Created:	04-01-2006 13:30
//
// Usage: WebUpdate [/install][/update] package pifURL
//
// WebUpdate retrieves the package info from pifURL (local or remote) and
// installs or updates the package "package". Options:
//
//	/install	- install the package always
//	/update		- install the package only if present and outdated
//
// URL may be any URL that is recognized by the Win32 API InternetCrackUrl():
//
//	file://machine/share/updates/wizupdate.txt
//	ftp://ftp.acme.com/public/wizupdate.txt
//	http://www.acme.com/download/wizupdate.txt
//	https://secure.acme.com/download/wizupdate.txt
//
// You can test this sample as-is by specifying the following URL:
//
//	http://www.tarma.com/download/update.txt

#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <tchar.h>
#include <windows.h>
#pragma hdrstop

#include "..\..\Include\TWU.h"

//---------------------------------------------------------------------------
// Global variables

static BOOL g_bInstall;
static BOOL g_bUpdate;

//---------------------------------------------------------------------------
// Print usage summary

static void PrintUsage(const TCHAR *pszArg)
{
	if (pszArg != NULL && *pszArg != 0)
		_tprintf(_T("Unknown option \"%s\"\n"), pszArg);

	_tprintf(_T("Usage: WebUpdate [/install][/update] package pifURL\n\n")
		_T("WebUpdate retrieves the package info from the URL and installs or updates\n")
		_T("the package \"package\". If neither /install nor /update is given, packages\n")
		_T(" are merely listed without being downloaded or installed. Options:\n\n")
		_T("  /install  - install the packages always\n")
		_T("  /update   - install the packages only if present and outdated\n")
		_T("\nYou can test this sample as-is by specifying the following URL:\n\n")
		_T("  http://www.tarma.com/download/update.txt\n")
	);
}

//---------------------------------------------------------------------------
// General/progress callback

static URESULT TWUAPI _WUCallback(
	TWUHANDLE hSession,					// [In] Active session handle
	int nEventCode,						// [In] Event code (see below)
	const TCHAR *pszEventMsg,			// [In] Event message text (optional)
	UINT64 ulProgress,					// [In] Progress in operation
	UINT64 ulTotal,						// [In] (Estimated) size of operation
	LPARAM lCallbackCookie				// [In] Associated client cookie
)
{
	UNUSED(hSession);
	UNUSED(nEventCode);
	UNUSED(ulProgress);
	UNUSED(ulTotal);
	UNUSED(lCallbackCookie);

	_putts(pszEventMsg);
#ifdef _DEBUG
	switch (nEventCode)
	{
	default:
	case TWU_CBEVENT_SESSION_OPENING:
	case TWU_CBEVENT_SESSION_OPENERROR:
	case TWU_CBEVENT_SESSION_OPENED:
	case TWU_CBEVENT_SESSION_CLOSING:
	case TWU_CBEVENT_SESSION_CLOSED:
	case TWU_CBEVENT_CONNECTING:
	case TWU_CBEVENT_CONNECTERROR:
	case TWU_CBEVENT_CONNECTED:
		break;

	case TWU_CBEVENT_URL_OPENING:
	case TWU_CBEVENT_URL_OPENERROR:
	case TWU_CBEVENT_URL_OPENED:
	case TWU_CBEVENT_URL_CLOSED:
	case TWU_CBEVENT_FILE_OPENING:
	case TWU_CBEVENT_FILE_OPENERROR:
	case TWU_CBEVENT_FILE_OPENED:
	case TWU_CBEVENT_FILE_CLOSED:
		_tprintf(_T("\tURL=%s\n\tFile=%s\n"),
			TWUGetEventArg(pszEventMsg, 0),
			TWUGetEventArg(pszEventMsg, 1));
		break;

	case TWU_CBEVENT_FILE_DELETING:
	case TWU_CBEVENT_FILE_DELETED:
	case TWU_CBEVENT_FILE_DELETEERROR:
		_tprintf(_T("\tFile=%s\n"),
			TWUGetEventArg(pszEventMsg, 0));
		break;

	case TWU_CBEVENT_DOWNLOAD_START:
	case TWU_CBEVENT_DOWNLOAD_END:
	case TWU_CBEVENT_DOWNLOAD_READERROR:
	case TWU_CBEVENT_DOWNLOAD_WRITEERROR:
	case TWU_CBEVENT_DOWNLOAD_PROGRESS:
		_tprintf(_T("\tURL=%s\n\tFile=%s\n"),
			TWUGetEventArg(pszEventMsg, 0),
			TWUGetEventArg(pszEventMsg, 1));
		break;

	case TWU_CBEVENT_PACKAGE_ENUM:
		_tprintf(_T("\tPackage=%s\n"),
			TWUGetEventArg(pszEventMsg, 0));
		break;

	case TWU_CBEVENT_PROCESS_STARTING:
	case TWU_CBEVENT_PROCESS_STARTED:
	case TWU_CBEVENT_PROCESS_ERROR:
	case TWU_CBEVENT_PROCESS_TERMINATED:
		_tprintf(_T("\tExecutable=%s\n"),
			TWUGetEventArg(pszEventMsg, 0));
		break;

	case TWU_CBEVENT_PIF_STARTPARSE:
	case TWU_CBEVENT_PIF_ENDPARSE:
		_tprintf(_T("\tPIF=%s\n"),
			TWUGetEventArg(pszEventMsg, 0));
		break;

	case TWU_CBEVENT_PIF_STARTPACKAGE:
	case TWU_CBEVENT_PIF_ENDPACKAGE:
		_tprintf(_T("\tPIF=%s\n\tSection=%s\n"),
			TWUGetEventArg(pszEventMsg, 0),
			TWUGetEventArg(pszEventMsg, 1));
		break;
	}
#endif
	return ERROR_SUCCESS;
}

//---------------------------------------------------------------------------
// Enumeration callback that downloads required packages. It sets
// pPackage->lUserCookie1 to nonzero if successful.

static URESULT TWUAPI _WUDownloadCB(
	TWUHANDLE hSession,					// [In] Active session handle
	TWUPackageData *pPackage,			// [In] Current package
	LPARAM lCookie						// [In] Client cookie
)
{
	URESULT uResult;
	BOOL bDownload;

	UNUSED(lCookie);

	uResult = ERROR_SUCCESS;	// To continue enumeration
	bDownload = FALSE;

	_tprintf(
		_T("Package %s: Available version=%u.%u.%u.%u\n"),
		pPackage->szPackageName,
		HIWORD(pPackage->dwVersionHigh), LOWORD(pPackage->dwVersionHigh),
		HIWORD(pPackage->dwVersionLow), LOWORD(pPackage->dwVersionLow));

	// Find out what to do, depending on command line options
	if (g_bInstall)
		bDownload = TRUE;		// Always download the package
	else if (g_bUpdate)
	{
		TWUInstalledInfo ii;

		// Get information about the currently installed version
		ii.uStructSize = sizeof(TWUInstalledInfo);
		uResult = TWUFindInstalledInfo(pPackage, &ii);
		if (uResult == ERROR_SUCCESS)
		{
			_tprintf(
				_T("Package %s: Installed version=%u.%u.%u.%u\n"),
				pPackage->szPackageName,
				HIWORD(ii.dwVersionHigh), LOWORD(ii.dwVersionHigh),
				HIWORD(ii.dwVersionLow), LOWORD(ii.dwVersionLow));
			bDownload = TWUCompareVersions(ii.dwVersionLow, ii.dwVersionHigh,
				pPackage->dwVersionLow, pPackage->dwVersionHigh) < 0;
		}
		uResult = ERROR_SUCCESS;
	}

	// Download the package installer if required and mark the package
	// if successful.
	if (bDownload)
	{
		_tprintf(_T("Downloading package %s: %s...\n"),
			pPackage->szPackageName, pPackage->szInstallerPath);

		uResult = TWUGetPackageInstaller(hSession, NULL, pPackage);
		if (uResult == ERROR_SUCCESS) pPackage->lUserCookie1 = TRUE;

		_tprintf(_T("Downloaded package %s: %s; result=%u\n"),
			pPackage->szPackageName, pPackage->szDownloadPath, uResult);
	}
	else
	{
		_tprintf(_T("Package %s does not need to be downloaded\n"),
			pPackage->szPackageName);
	}
	return uResult;
}

//---------------------------------------------------------------------------
// Enumeration callback that installs required packages

static URESULT TWUAPI _WUInstallCB(
	TWUHANDLE hSession,					// [In] Active session handle
	TWUPackageData *pPackage,			// [In] Current package
	LPARAM lCookie						// [In] Client cookie
)
{
	URESULT uResult;
	DWORD dwExitCode;

	UNUSED(lCookie);

	uResult = ERROR_SUCCESS;	// To continue enumeration

	// Install the package if it was successfully downloaded
	if (pPackage->lUserCookie1)
	{
		_tprintf(_T("Installing package %s: %s...\n"),
			pPackage->szPackageName, pPackage->szDisplayName);

		uResult = TWURunPackageInstaller(hSession, pPackage, NULL, NULL,
			TWU_RUNF_SYNCHRONOUS, NULL, &dwExitCode);

		_tprintf(_T("Installed package %s: %s; result=%u, exit code=%u (0x%08X)\n"),
			pPackage->szPackageName, pPackage->szDownloadPath, uResult,
			dwExitCode, dwExitCode);
	}
	return uResult;
}

//---------------------------------------------------------------------------
// Main entry point

int _tmain(int argc, LPTSTR argv[])
{
	TCHAR szPackageName[64], szURL[1024], szMessage[256];
	TWUHANDLE hSession;
	TWUPackageData *pPackage, *pFailedPackage;
	URESULT uResult;
	int nResult, i;

	nResult = 0;
	szPackageName[0] = szURL[0] = 0;

	// Parse the command line arguments; argv[0] = command name
	for (i = 1; i < argc; ++i)
	{
		if (argv[i][0] == _T('/') || argv[i][0] == _T('-'))
		{
			// Option
			if (lstrcmpi(argv[i] + 1, _T("install")) == 0)
			{
				g_bInstall = TRUE;
				g_bUpdate = FALSE;
			}
			else if (lstrcmpi(argv[i] + 1, _T("update")) == 0)
			{
				g_bInstall = FALSE;
				g_bUpdate = TRUE;
			}
			else
			{
				PrintUsage(argv[i]);
				return -1;
			}
		}
		else
		{
			// Package name or URL
			if (szPackageName[0] == 0)
				lstrcpyn(szPackageName, argv[i], __countof(szPackageName));
			else if (szURL[0] == 0)
				lstrcpyn(szURL, argv[i], __countof(szURL));
			else
			{
				PrintUsage(argv[i]);
				return -1;
			}
		}
	}

	// Did we get a package name and URL?
	if (szPackageName[0] == 0 || szURL[0] == 0)
	{
		PrintUsage(NULL);
		return -1;
	}

	// Retrieve and parse the package information file
	_tprintf(_T("Retrieving package information from %s...\n"), szURL);
	uResult = TWUOpenUpdateURL(szURL, _WUCallback, 0, &hSession);
	if (uResult != ERROR_SUCCESS)
	{
		TWUGetErrorMessage(uResult, szMessage, __countof(szMessage));
		_tprintf(_T("Error %u while opening URL %s: %s\n"),
			uResult, szURL, szMessage);
		return 1;
	}

	// Find the desired package
	pPackage = TWUFindPackage(hSession, szPackageName);
	if (pPackage == NULL)
	{
		_tprintf(_T("Package %s not found at URL %s\n"),
			szPackageName, szURL);
		nResult = 2;
		goto lblExit;
	}

	// First pass: Download any packages that must be downloaded
	pFailedPackage = TWUEnumPackageDependencies(hSession, pPackage,
		_WUDownloadCB, 0, &uResult);
	if (pFailedPackage != NULL || uResult != ERROR_SUCCESS)
	{
		TWUGetErrorMessage(uResult, szMessage, __countof(szMessage));
		_tprintf(_T("Error %u while downloading package %s: %s\n"),
			uResult, pFailedPackage ? pFailedPackage->szPackageName :
			szPackageName, szMessage);
		nResult = 3;
		goto lblExit;
	}

	// Second pass: Install any packages that must be installed
	pFailedPackage = TWUEnumPackageDependencies(hSession, pPackage,
		_WUInstallCB, 0, &uResult);
	if (pFailedPackage != NULL || uResult != ERROR_SUCCESS)
	{
		TWUGetErrorMessage(uResult, szMessage, __countof(szMessage));
		_tprintf(_T("Error %u while installing package %s: %s\n"),
			uResult, pFailedPackage ? pFailedPackage->szPackageName :
			szPackageName, szMessage);
		nResult = 4;
		goto lblExit;
	}

lblExit:
	//-----	Clean up
	TWUCleanupDownloads(hSession);
	TWUCloseSession(hSession);
	return nResult;
}
